-- https://forum.dcs.world/topic/90202-lua-scripting-radio-commands-f10/

local spawnedGroups = {}

local id = 1000

function removeAllPlanes()
	for i,planeGroup in ipairs(spawnedGroups) do
		for i,plane in ipairs(planeGroup:GetUnits()) do
			plane:Destroy()
		end
	end
end

function buildGroup(loadoutID, num, aspect, skill)
	local groupTemp = UTILS.DeepCopy(GROUP:FindByName(loadoutID):GetTemplate()) -- need to copy (I dont think its copying)
	if (aspect == "Passive") then 
		groupTemp["CountryID"] = 80
		groupTemp["SpawnCoalitionID"] = 2
	end
	groupTemp["name"] = "Group" .. getID()
	groupTemp["units"][1]["skill"] = skill
	groupTemp["units"][1]["name"] = "Plane" .. getID()
	local planeTemp = UTILS.DeepCopy(groupTemp["units"][1])
	for i = 2, num do
		local cp = UTILS.DeepCopy(planeTemp)
		cp["y"] = cp["y"] + (500 * (i - 1))
		cp["x"] = cp["x"] + (500 * (i - 1))
		cp["name"] = "Plane" .. getID()
		cp["skill"] = skill
		table.insert(groupTemp["units"], cp)
	end

	return SPAWN:NewFromTemplate(groupTemp, groupTemp["name"], groupTemp["name"])
end

function getID()
	id = id + 1
	return tostring(id)
end

function spawnGroup(playerGroup, spawnObject, mach, alt, dist, aspect, data)

	local Player = playerGroup:GetUnit(1)

	local Position = Player:GetPointVec3()
	local Heading = Player:GetHeading()
	local spawnHeading = Heading
	
	local speed = 0
	if (mach == "co") then
		speed = Player:GetVelocityMPS()
	else
		local temp = Position:GetTemperature()
		local speedOfSound = 331.3 * math.sqrt(1+ temp/273.15)
		speed = mach * speedOfSound
	end

	if (aspect == "Offensive") then
		Position:SetX(Position:GetX() + (math.cos(math.rad(Heading + 2)) * UTILS.FeetToMeters(3000)))
		Position:SetZ(Position:GetZ() + (math.sin(math.rad(Heading + 2)) * UTILS.FeetToMeters(3000)))
		speed = speed --+ 50
	elseif (aspect == "Defensive") then
		Position:SetX(Position:GetX() - (math.cos(math.rad(Heading + 2)) * UTILS.FeetToMeters(3000)))
		Position:SetZ(Position:GetZ() - (math.sin(math.rad(Heading + 2)) * UTILS.FeetToMeters(3000)))
		speed = speed --- 50
	elseif (aspect == "Passive") then
		Position:SetX(Position:GetX() + (math.cos(math.rad(Heading + 2)) * UTILS.FeetToMeters(3000)))
		Position:SetZ(Position:GetZ() + (math.sin(math.rad(Heading + 2)) * UTILS.FeetToMeters(3000)))
		speed = speed - 20
	else
		Position:SetX(Position:GetX() + (math.cos(math.rad(Heading)) * UTILS.FeetToMeters(dist*6076.12)))
		Position:SetZ(Position:GetZ() + (math.sin(math.rad(Heading)) * UTILS.FeetToMeters(dist*6076.12)))
		spawnHeading = (Heading + 180) % 360
	end
	
	if (alt ~= "co") then
    	Position:SetY(Position:GetY() + UTILS.FeetToMeters(alt))
	end

	local spawnedPlane = spawnObject:InitHeading(spawnHeading):SpawnFromPointVec3(Position)
	
	if (aspect == "Passive") then
		if (data.passiveType == "Straight") then
			local desto = Player:GetPointVec3()
			desto:SetX(Position:GetX() + (math.cos(math.rad(Heading)) * UTILS.FeetToMeters(300000)))
			desto:SetZ(Position:GetZ() + (math.sin(math.rad(Heading)) * UTILS.FeetToMeters(300000)))
			spawnedPlane:RouteAirTo(desto, POINT_VEC3.RoutePointAltType.Baro, POINT_VEC3.RoutePointType.TurningPoint, POINT_VEC3.RoutePointAction.TurningPoint, speed)
		elseif (data.passive == "Circle") then
			local desto = Player:GetPointVec3()
			desto:SetX(Position:GetX() + (math.cos(math.rad(Heading)) * UTILS.FeetToMeters(10000)))
			desto:SetZ(Position:GetZ() + (math.sin(math.rad(Heading)) * UTILS.FeetToMeters(10000)))
			spawnedPlane:TaskOrbit(desto, Position:GetY(), speed)
		else
			--error
		end
	end

	spawnedPlane:OptionRTBAmmo(0):OptionRTBBingoFuel(false):OptionROEWeaponFree()
	spawnedPlane:TaskAttackGroup(playerGroup)
	-- spawnedPlane:KnowUnit(Player, true, true)

	-- SCHEDULER:New(nil, function()
	-- 	MESSAGE:New((spawnedPlane:IsUnitDetected(Player)[1])):ToAll()
	--   end, {}, 0, 2)
	
	table.insert(spawnedGroups, spawnedPlane)
end

function buildLoadoutName(enemyGroup)
	local skill = enemyGroup:GetSkill()
	local skillStr = "" --skill .. " - "

	local ammo = enemyGroup:GetUnit(1):GetAmmo()
	local weapons = {}
	if (ammo ~= nil) then
		for i = 1, #ammo do
			local dn = ammo[i].desc.displayName
			if (dn ~= nil) then
				if (ammo[i].desc.missileCategory ~= nil) then -- maybe use category? category = 0 = gun? category = 1 = missile?
					weapons[dn] = ammo[i].count
				else
					weapons["Guns"] = 0
				end
			else
				MESSAGE:New(UTILS.OneLineSerialize(ammo[i])):ToAll()
			end
		end
		if (tablelength(weapons) == 1 and weapons["Guns"] == 0) then
			return skillStr .. "Guns"
		elseif (tablelength(weapons) ~= 0) then
			local str = skillStr
			for k, v in pairs(weapons) do
				if (k ~= "Guns") then
					str = str.. tostring(v) .. "x " .. k .. " "
				end
			end
			return str
		else
			return skillStr .. "Empty"
		end
	else
		return skillStr .. "Empty"
	end
end

function tablelength(T)
	local count = 0
	for _ in pairs(T) do count = count + 1 end
	return count
end

-- http://lua-users.org/wiki/CopyTable
function deep_copy(t)
	local result = {}
	for key, value in pairs(t) do
		if type(value) == "table" then
			result[key] = deep_copy(value)
		else
			result[key] = value
		end
	end
	return result
end